!### macros #####################################################
!
#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
!
#include "tm5.inc"
!

! This module defines the tracers for a "forward" CarbonTracker
! simulation, in which components of the carbon cycle are transported
! as independent tracers.

module chem_param

  use GO, only : gol, goPr, goErr
  
  use binas, only : xmair
!  use reaction_data
#ifndef without_photolysis
  use photolysis_data
#endif
  use deposition_data

  implicit none

  !
  ! definition of the chemistry: #reactions, order of species, etc.
  ! parameters needed for chemistry and rate constants 
  !

  ! nmark: number of 'marked' tracers 
  integer, parameter :: nmark = 1
  character(len=8),dimension(nmark) :: marknam = (/ 'test    ' /)

  ! ntrace: number of tracers for chemistry
  integer, parameter :: ntrace  = 150
  ! ntracet: number of transported tracers
  integer, parameter :: ntracet = 150
  ! ntrace_chem: non-transported tracers   
  integer, parameter :: ntrace_chem  = ntrace-ntracet
  ! maxtrace: total number of tracers
  integer, parameter :: maxtrace = ntrace + 0 

  integer, parameter :: ntrace_emis = 4
  integer, parameter :: nmembersloc = 5
  integer, parameter :: tracer_name_len = 8
  !
  ! additional fields used in chemistry routine alone 
  ! (more meteo-like files in units different from #/cm3)
  !
  integer,parameter                   :: n_extra = 0
  integer,parameter                   :: nstd    = 1
  integer, dimension(nstd), parameter :: istd    = (/ 1 /)
  !
  ! species name
  !
  character(len=8), dimension(maxtrace) :: names  = (/'co2_001','co2_002','co2_003','co2_004','co2_005','co2_006','co2_007','co2_008','co2_009','co2_010','co2_011','co2_012','co2_013','co2_014','co2_015','co2_016','co2_017','co2_018','co2_019','co2_020','co2_021','co2_022','co2_023','co2_024','co2_025','co2_026','co2_027','co2_028','co2_029','co2_030','co2_031','co2_032','co2_033','co2_034','co2_035','co2_036','co2_037','co2_038','co2_039','co2_040','co2_041','co2_042','co2_043','co2_044','co2_045','co2_046','co2_047','co2_048','co2_049','co2_050','co2_051','co2_052','co2_053','co2_054','co2_055','co2_056','co2_057','co2_058','co2_059','co2_060','co2_061','co2_062','co2_063','co2_064','co2_065','co2_066','co2_067','co2_068','co2_069','co2_070','co2_071','co2_072','co2_073','co2_074','co2_075','co2_076','co2_077','co2_078','co2_079','co2_080','co2_081','co2_082','co2_083','co2_084','co2_085','co2_086','co2_087','co2_088','co2_089','co2_090','co2_091','co2_092','co2_093','co2_094','co2_095','co2_096','co2_097','co2_098','co2_099','co2_100','co2_101','co2_102','co2_103','co2_104','co2_105','co2_106','co2_107','co2_108','co2_109','co2_110','co2_111','co2_112','co2_113','co2_114','co2_115','co2_116','co2_117','co2_118','co2_119','co2_120','co2_121','co2_122','co2_123','co2_124','co2_125','co2_126','co2_127','co2_128','co2_129','co2_130','co2_131','co2_132','co2_133','co2_134','co2_135','co2_136','co2_137','co2_138','co2_139','co2_140','co2_141','co2_142','co2_143','co2_144','co2_145','co2_146','co2_147','co2_148','co2_149','co2_150'/)

  integer, parameter :: ico2bg=-1      ! background (initial condition) co2 mixing ratio
  integer, parameter :: ico2ff=-1      ! co2 mixing ratio due to fossil fuel fluxes
  integer, parameter :: ico2ocean=-1   ! co2 mixing ratio due to air-sea exchange
  integer, parameter :: ico2bio=-1     ! co2 mixing ratio due to terrestrial biosphere fluxes excluding fires
  integer, parameter :: ico2fires=-1   ! co2 mixing ratio due to wildfire emissions

  ! 
  ! some stuff for the budgets....
  !
!  integer, parameter           :: njnum = 1
!  integer, parameter           :: nreac = 1
!  integer, parameter           :: nhet = 1
!  integer, parameter           :: nreacw = 1
!  character(len=8), dimension(njnum)  :: jnam   = (/'dummy   '/)
!  character(len=8), dimension(nreac)  :: ratnam = (/'dummy   '/)
!  character(len=8), dimension(nreacw) :: rwnam  = (/'dummy   '/)
!  character(len=8), dimension(nhet) :: rhetnam  = (/'dummy   '/)

  ! molar weights of components

  real,    parameter :: xmh=1.0079
  real,    parameter :: xmn=14.0067
  real,    parameter :: xmc=12.01115
  real,    parameter :: xms=32.064
  real,    parameter :: xmo=15.9994
  real,    parameter :: xmcl=35.453
  real,    parameter :: xmf=18.9984032
  real,    parameter :: xmh2o=xmh*2+xmo
  real,    parameter :: xmco2=xmc+2*xmo
  real,    parameter :: xmrn222=222.
  real,    parameter :: xmpb210=210.
  real,    parameter :: xmsf6= xms+6*xmf
  real,    parameter :: xmnmv=999. ! this is a dummy molecular mass

  real,dimension(maxtrace),parameter :: ra=xmco2
  real,dimension(maxtrace),parameter      :: fscale = xmair/ra
  real,dimension(maxtrace),parameter      :: uscale = 1.e6

  !   fscale(ntrace): scaling factor for conversion of mixing ratios 
  !   in kg tracer per kg air to practical mixing ratio units (e.g. ppm)
  !   In this version: ratio of molecular weight  of tracer to that of air 


  ! ********************************************************************
  ! rates
  ! ********************************************************************

  ! heterogeneous removal rates
!  real                    :: henry(ntracet,ntemp)


  ! ********************************************************************
  ! dummy tracers
  ! ********************************************************************

  ! dummy indices for tracers not included in this chemistry;
  ! comparision of tracer index with this numbers will fail ...
  
  integer, parameter   ::  iso2  = -999
  integer, parameter   ::  iso4  = -998
  integer, parameter   ::  inh3  = -997
  integer, parameter   ::  ihno3 = -996
  integer, parameter   ::  ino   = -995
  integer, parameter   ::  ino2  = -994
  integer, parameter   ::  io3   = -993
  integer, parameter   ::  ihno4 = -992
  integer, parameter   ::  ih2o2 = -991


  ! ********************************************************************
  ! aerosol
  ! ********************************************************************

  real, parameter         :: density_ref = 1800.0   ! for 'reference' density calculations


  ! ********************************************************************
  ! dry deposition
  ! ********************************************************************

  ! number of tracers on which dry depositions should be applied:
  integer, parameter  ::  ndep = 0

  ! tracer indices on which dry depositions should be applied:
  integer, parameter  ::  idep(1) = (/ -999 /)

  ! deposition parameters:
  
  real, parameter  ::  ddep_diffrb(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rsoil(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rwat(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rws(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rsnow(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rmes(1) = (/ -999.9 /)

  real, parameter  ::  ddep_rcut(1) = (/ -999.9 /)

  real, parameter  ::  ddep_diffcf(1) = (/ -999.9 /)

  ! some vd fields are copies:
  integer, parameter  ::  vd_ncopy = 0   ! dummy
  integer, parameter  ::  vd_copy_itarget(1) = (/ -999 /)
  integer, parameter  ::  vd_copy_isource(1) = (/ -999 /)


  ! number of aerosol bins used for deposition:
  integer, parameter  ::  nrdep = 0
  
  ! aerosol radii used for each bin:
  real, parameter     ::  lur(1) = (/ -999.9 /)


  ! ********************************************************************
  ! wet deposition
  ! ********************************************************************

  !
  ! nscav       : selected species for scavenging
  ! nscav_index : index for scavenging:
  ! nscav_type  : type of scavenging:
  !               0 no scavenging
  !               1 scavenging 100 % solubility assumed
  !               2 scavenging henry solubility assumed
  !               3 scavenging, aerosol removal assumed
  !               4 scavenging, special case for SO2 with aq phase diss.
  !

  integer, parameter  ::  nscav = 0

  integer, parameter  ::  nscav_index(1) = (/ -999 /)

  ! note CMK CFD: wetS accounts for removal of SO2, so nscav_type = 0
  !               nh3 is taken out anyhow, since most will be scavenged
  !               by acidic falling frops below cloud
  integer, parameter  :: nscav_type(1) =  (/ -999 /)


end module chem_param
